#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.Design;
using System.Windows.Forms;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    ///  Use the Mouse to drag the chart in any direction allowing you to pan around on the chart. Dis/Enable by clicking the toolbar text "Pan"
    /// </summary>
	/// 
	/// Adapted from tulanch MousePan, which is an apaptation of Devdas's code ScrollsLite obtained from elite section on BMT forum
	/// 
	/// 
    [Description("Deploy in chart panel. With mouse positioned within that chart panel, press control key and left mouse button - the drag chart in any direction allowing you to pan around on the chart. Do not select any indicator or bar item when dragging - perform left mouse click on open background. Adapted from Devdas's ScrollsLite code")]
    public class MousePanToolbar : Indicator
    {
        #region Variables
 
			private double     delta_margin;	// amount to adjusted y axis min max values (price tick units)
		    private int        ct;				// clip region top (pixels)
		    private int        cb;              // clip region bottom (pixels)
		    private int        old_mouse_y;     // holds mouse location (pixels)
		    private double     scale;           // Price ticks per Pixel
			int temp_y;
		
			private bool ChartPanOn  = true;
			
			// Pawnbroker - Add toolbar button to toggle pan on and off
			private ToolStrip strip 		= null;
			private ToolStripButton button 	= null;
			private  ToolStripSeparator myToolStripSeparator;
		
			private string versionNumber = "1.00";
		
		#endregion

 		// ////////////////////////////////////////////////
		// initialize is called upon every initial 1st draw
		// ////////////////////////////////////////////////
        protected override void Initialize()
        {
            Overlay				= true;
			CalculateOnBarClose = false;
			AutoScale           = true;
			PaintPriceMarkers 	= true;
        }

 		// //////////////////////////// 
		// onstartup is called one time 
		// //////////////////////////// 
		protected override void OnStartUp()
		{
			base.OnStartUp();
			
 			this.ChartControl.ChartPanel.MouseUp   += new MouseEventHandler(MouseDrag_OnScale);
 			this.ChartControl.ChartPanel.MouseDown += new MouseEventHandler(MouseDown_OnChart);
			this.ChartControl.ChartPanel.MouseMove += new MouseEventHandler(MouseDrag_OnChart);
			
			Control[] tscontrol = ChartControl.Controls.Find("tsrTool",false);
			if(tscontrol.Length > 0)
			{
			  button = new ToolStripButton("Pan");	
			  button.Click += hideshow;
			  myToolStripSeparator 	= new ToolStripSeparator ();
			  strip = (ToolStrip) tscontrol[0];
			  strip.Items.Add(myToolStripSeparator);
			  strip.Items.Add(button);
			  button.ForeColor = Color.Blue;
			}
			
		}
		
 
		// ////////////////////
		// for every bar update
		// ////////////////////
        protected override void OnBarUpdate()
       {
		 // does nothing but is required
       }
	
		#region HideShow methods
		private void hideshow(object s, EventArgs e)
		{
			if ( ChartPanOn == false ) {
				ChartPanOn = true;
				button.ForeColor = Color.Blue;
			}
			else
			{
				ChartPanOn = false;
				button.ForeColor = Color.Silver;
			}
		}
		#endregion

		
		// ////////////////////////////////////////////////////////////////////////////
		// Plot function offers access to clipping bounds - pixel for the current panel
		// ////////////////////////////////////////////////////////////////////////////
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
        {
        	if(ChartControl != null)
			{
				
				// ////////////////////////////////
				// calculates Price Ticks per Pixel 
				// ////////////////////////////////				
				ct = (int)graphics.ClipBounds.Top;
				cb = (int)graphics.ClipBounds.Bottom ;
				scale = (max - min)/ ( cb - ct );
        	}
		}		
		
		
 		// /////////////////////////////////////////////////////// 
		// corrects Price Ticks per Pixel when y axis scale adjust
		// /////////////////////////////////////////////////////// 
		private void MouseDrag_OnScale(object sender, MouseEventArgs e)
		{
			
			if( this.ChartControl.CanvasRight < e.X  && e.X < this.ChartControl.Size.Width)
			{
				if(e.Button == MouseButtons.Left)
				{
					// //////////////////////////////////////
					// invokes Plot call to recalculate scale
					// //////////////////////////////////////
					this.ChartControl.ChartPanel.Invalidate(); 
				}
			}
		}
 
		
		// ///////////////////////////////////////////// 
		// updates the mouse location used in drag logic 
		// ///////////////////////////////////////////// 
 		private void MouseDown_OnChart(object sender, MouseEventArgs e)
		{
			if( !ChartPanOn)
			{
				return;
			}
			
			if(e.Button == MouseButtons.Left)
			{
				if(!MouseOnChart(e.X,e.Y))
				{
					return;
				}
				else
				{
					old_mouse_y = e.Y;
				}
			}
		}
 		
		// ///////////////////////////////
		// this moves the view winow/chart 
		// ///////////////////////////////
		private void MouseDrag_OnChart(object sender, MouseEventArgs e)
		{
			
			if( !ChartPanOn)
			{
				return;
			}
			
			if(e.Button == MouseButtons.Left)
			{
				if(!MouseOnChart(e.X,e.Y))
				{
					return;
				}
				else
				{
					
					// ////////////////////////////////////////////////////////////
					// forces Fixed mode - watch for f icon upper right hand corner
					// ////////////////////////////////////////////////////////////
					this.ChartControl.YAxisRangeTypeRight = YAxisRangeType.Fixed;
					
					// //////////////////////////////////////////////////////////////////////////////// 
					// scales the FixedPanel bounds offset based delta mouse movement defined in pixels 
					// FixedPanel units are in ticks
					// mouse units are in inverted pixel units
					// scale is FixedPanelUnits per Pixel
					// //////////////////////////////////////////////////////////////////////////////// 
					delta_margin = (double) ( e.Y - old_mouse_y)*scale;
					this.ChartControl.FixedPanelMinRight = this.ChartControl.FixedPanelMinRight + delta_margin;
					this.ChartControl.FixedPanelMaxRight = this.ChartControl.FixedPanelMaxRight + delta_margin;
 					
					old_mouse_y = e.Y;
				
					// ///////////////////////////////////////////////
					// Note X axis moves based on default NT movemenet
					// ///////////////////////////////////////////////
					
					this.ChartControl.ChartPanel.Invalidate(); // repaints the screen 
					
				}
 			}


			
		}
		
		// /////////////////////////////////////////////
		// utility to see if mouse location is  on chart
		// /////////////////////////////////////////////
		protected bool MouseOnChart(int x, int y)
		{
			// ////////////////////////////////////////////////////
			// validates mouse location only if control key pressed 
			// ////////////////////////////////////////////////////
 			if(!ChartPanOn)
 			{
				return false;
			}
			
			// restricts movemenet to panel this indicator in - typically the price panel 
			temp_y = this.ChartControl.GetYByValue(this,this.ChartControl.FixedPanelMinRight);  
			
			if( x < this.ChartControl.CanvasLeft || x >= this.ChartControl.CanvasRight || y < this.ChartControl.CanvasTop || y > temp_y)
			{
				return false;
			}
			else 
			{
				return true;
			}
		}
		
		// /////////////////
		// release handelers
		// /////////////////
		protected override void OnTermination()
		{
 			this.ChartControl.ChartPanel.MouseUp   -= MouseDrag_OnScale;
			this.ChartControl.ChartPanel.MouseDown -= MouseDown_OnChart;
			this.ChartControl.ChartPanel.MouseMove -= MouseDrag_OnChart;  
			
			// Pawnbroker Remove toolstrip
		   	if((strip != null) && (button != null))
			{
		        strip.Items.Remove(button);
				strip.Items.Remove(myToolStripSeparator);
			}
			strip = null;
			button = null;
			myToolStripSeparator = null;

			base.OnTermination();
		}
		
		#region Properties
		[Description("Version Number")]
		[GridCategory("Parameters")]
		[ReadOnlyAttribute(true)]
		public string VersionNumber
		{
			get { return versionNumber; }
			set { ;}
		}
		#endregion
    }
}




#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private MousePanToolbar[] cacheMousePanToolbar = null;

        private static MousePanToolbar checkMousePanToolbar = new MousePanToolbar();

        /// <summary>
        /// Deploy in chart panel. With mouse positioned within that chart panel, press control key and left mouse button - the drag chart in any direction allowing you to pan around on the chart. Do not select any indicator or bar item when dragging - perform left mouse click on open background. Adapted from Devdas's ScrollsLite code
        /// </summary>
        /// <returns></returns>
        public MousePanToolbar MousePanToolbar(string versionNumber)
        {
            return MousePanToolbar(Input, versionNumber);
        }

        /// <summary>
        /// Deploy in chart panel. With mouse positioned within that chart panel, press control key and left mouse button - the drag chart in any direction allowing you to pan around on the chart. Do not select any indicator or bar item when dragging - perform left mouse click on open background. Adapted from Devdas's ScrollsLite code
        /// </summary>
        /// <returns></returns>
        public MousePanToolbar MousePanToolbar(Data.IDataSeries input, string versionNumber)
        {
            if (cacheMousePanToolbar != null)
                for (int idx = 0; idx < cacheMousePanToolbar.Length; idx++)
                    if (cacheMousePanToolbar[idx].VersionNumber == versionNumber && cacheMousePanToolbar[idx].EqualsInput(input))
                        return cacheMousePanToolbar[idx];

            lock (checkMousePanToolbar)
            {
                checkMousePanToolbar.VersionNumber = versionNumber;
                versionNumber = checkMousePanToolbar.VersionNumber;

                if (cacheMousePanToolbar != null)
                    for (int idx = 0; idx < cacheMousePanToolbar.Length; idx++)
                        if (cacheMousePanToolbar[idx].VersionNumber == versionNumber && cacheMousePanToolbar[idx].EqualsInput(input))
                            return cacheMousePanToolbar[idx];

                MousePanToolbar indicator = new MousePanToolbar();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.VersionNumber = versionNumber;
                Indicators.Add(indicator);
                indicator.SetUp();

                MousePanToolbar[] tmp = new MousePanToolbar[cacheMousePanToolbar == null ? 1 : cacheMousePanToolbar.Length + 1];
                if (cacheMousePanToolbar != null)
                    cacheMousePanToolbar.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheMousePanToolbar = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Deploy in chart panel. With mouse positioned within that chart panel, press control key and left mouse button - the drag chart in any direction allowing you to pan around on the chart. Do not select any indicator or bar item when dragging - perform left mouse click on open background. Adapted from Devdas's ScrollsLite code
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MousePanToolbar MousePanToolbar(string versionNumber)
        {
            return _indicator.MousePanToolbar(Input, versionNumber);
        }

        /// <summary>
        /// Deploy in chart panel. With mouse positioned within that chart panel, press control key and left mouse button - the drag chart in any direction allowing you to pan around on the chart. Do not select any indicator or bar item when dragging - perform left mouse click on open background. Adapted from Devdas's ScrollsLite code
        /// </summary>
        /// <returns></returns>
        public Indicator.MousePanToolbar MousePanToolbar(Data.IDataSeries input, string versionNumber)
        {
            return _indicator.MousePanToolbar(input, versionNumber);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Deploy in chart panel. With mouse positioned within that chart panel, press control key and left mouse button - the drag chart in any direction allowing you to pan around on the chart. Do not select any indicator or bar item when dragging - perform left mouse click on open background. Adapted from Devdas's ScrollsLite code
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MousePanToolbar MousePanToolbar(string versionNumber)
        {
            return _indicator.MousePanToolbar(Input, versionNumber);
        }

        /// <summary>
        /// Deploy in chart panel. With mouse positioned within that chart panel, press control key and left mouse button - the drag chart in any direction allowing you to pan around on the chart. Do not select any indicator or bar item when dragging - perform left mouse click on open background. Adapted from Devdas's ScrollsLite code
        /// </summary>
        /// <returns></returns>
        public Indicator.MousePanToolbar MousePanToolbar(Data.IDataSeries input, string versionNumber)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.MousePanToolbar(input, versionNumber);
        }
    }
}
#endregion
