#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;

using System.Windows.Forms;
#endregion

// This namespace holds all indicators and is required. Do not change it.
// Credit to NT Forum and NT for the original indicator
// Revised by Aligator to include additional butons for Snap Mode, and Stay in Draw Mode - October 5, 2012
// Revised by aligator to include code to prevent false indicator termination on low bar numbers.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("Enter the description of your new custom indicator here")]
    public class ToolbarShortcutV1 : Indicator
    {
        #region Variables
		
		string[] strSnapMode = {"btnmnuDisable", "btnmnuBar", "btnmnuPrice", "btnmnuBarAndPrice"};
		string[] strLines = {"btnmnuStayInDrawMode","btnmnuRuler", "btnmnuLine", "btnmnuRay", "btnmnuExtendedLine", "btnmnuArrowLine", "btnmnuHorizontalLine", "btnmnuVerticalLine" };
		string[] strFibs = {"btnmnuFibonacciRetracements",	"btnmnuFibonacciExtensions", "btnmnuFibonacciTimeExtension", "btnmnuFibonacciCircle"};
		string[] strChannels = {"btnmnuAndrewsPitchfork", "btnmnuGannFan", "btnregressionChannel", "btnmnuTrendChannel"};
		string[] strShapes = {"btnmmnuEllipse", "btnmnuRectangle", "btnmnuTriangle", "btnmnuArc", "btnmnuText"};
		string[] strMarkers = {"btnmnuArrowUp", "btnmnuArrowDown", "btnmnuDiamond", "btnmnuDot", "btnmnuSquare", "btnmnuTriangleUp", "btnmnuTriangleDown"};
		
		ToolStrip ts = null;
		
		bool showSnapMode = true;
		bool showLines = true;
		bool showFibs = true;
		bool showChannels = true;
		bool showShapes = true;
		bool showMarkers = true;
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Overlay				= true;
			ZOrder				= 1;
			BarsRequired		= 0;
        }

		protected override void OnStartUp()
		{
			
	
			
			if (!showSnapMode &&!showLines && !showFibs && !showChannels && !showShapes && !showMarkers) return;
			
			ts = new ToolStrip();
			ts.Name = "ts";
			ts.Dock = DockStyle.Left;
			
			AddButtons("btnmnuSnapMode");
			
			ChartControl.Controls.Add(ts);
			
			
			if (showSnapMode)
			{
				foreach (string snapMode in strSnapMode)
				{
					AddButtons(snapMode);
				}
				AddSeparator();
			}
			
			
			if (showLines)
			{
				foreach (string lines in strLines)
				{
					this.AddButtons(lines);
				}
				AddSeparator();
			}
			
			if (showFibs)
			{
				foreach (string fibs in strFibs)
				{
					AddButtons(fibs);
				}
				AddSeparator();
			}
			
			if (showChannels)
			{
				foreach (string channels in strChannels)
				{
					AddButtons(channels);
				}
				AddSeparator();
			}
			
			if (showShapes)
			{
				foreach (string shapes in strShapes)
				{
					AddButtons(shapes);
				}
				AddSeparator();
			}
			
			if (showMarkers)
			{
				foreach (string markers in strMarkers)
				{
					AddMarkers(markers);
				}
				AddSeparator();
			}
			
			
			AddButtons("btnmnuRemoveAllDrawObjects");
			
			ChartControl.Controls.Add(ts);
								
		}
		
		private void AddSeparator()
		{
			if (ChartControl == null || ts == null) return;
			
			ToolStripSeparator sep = new ToolStripSeparator();
			ts.Items.Add(sep);
		}

		
		private void AddSnapMode(string btnSnapMode)
		{
			if (ChartControl == null || ts == null) return;
			
			ToolStripDropDownItem btnDraw = (ToolStripDropDownItem)((ToolStripDropDownItem)((ToolStrip)ChartControl.Controls["tsrTool"]).Items["btnDraw"]).DropDownItems["mnuSnapMode"];
			if (btnDraw == null) return;		
			
			ToolStripItem item = btnDraw.DropDownItems[btnSnapMode.Replace("btn", string.Empty)];
			if (item == null) return;
			
			ToolStripButton btn = new ToolStripButton();
			btn.Name = btnSnapMode;
			btn.Text = btnSnapMode.Replace("btn", string.Empty).Replace("mmnu", string.Empty).Replace("mnu", string.Empty);
			btn.DisplayStyle = ToolStripItemDisplayStyle.Text;
			btn.Text = item.Text;
			btn.Click += new EventHandler(btn2_Click);
			
			ts.Items.Add(btn);
		
		}		
		private void AddButtons(string btnName)
		{
			if (ChartControl == null || ts == null) return;
			
			ToolStripDropDownItem btnDraw = (ToolStripDropDownItem)((ToolStrip)ChartControl.Controls["tsrTool"]).Items["btnDraw"];
			if (btnDraw == null) return;
			
			ToolStripItem item = btnDraw.DropDownItems[btnName.Replace("btn", string.Empty)];
			if (item == null) return;
			
			ToolStripButton btn = new ToolStripButton();
			btn.Name = btnName;
			btn.Text = btnName.Replace("btn", string.Empty).Replace("mmnu", string.Empty).Replace("mnu", string.Empty);
			btn.DisplayStyle = ToolStripItemDisplayStyle.Image;
			btn.Image = item.Image;
			btn.Click += new EventHandler(btn_Click);
			
			ts.Items.Add(btn);
			
		}
		
		private void AddMarkers(string btnMarker)
		{
			if (ChartControl == null || ts == null) return;
			
			ToolStripDropDownItem btnDraw = (ToolStripDropDownItem)((ToolStripDropDownItem)((ToolStrip)ChartControl.Controls["tsrTool"]).Items["btnDraw"]).DropDownItems["mnuChartMarker"];
			if (btnDraw == null) return;		
			
			ToolStripItem item = btnDraw.DropDownItems[btnMarker.Replace("btn", string.Empty)];
			if (item == null) return;
			
			ToolStripButton btn = new ToolStripButton();
			btn.Name = btnMarker;
			btn.Text = btnMarker.Replace("btn", string.Empty).Replace("mnu", string.Empty);
			btn.DisplayStyle = ToolStripItemDisplayStyle.Image;
			btn.Image = item.Image;
			btn.Click += new EventHandler(btn1_Click);
			
			ts.Items.Add(btn);
		
		}
		
		
		private void btn_Click(object sender, EventArgs e)
		{
			
			ToolStripItem item = (ToolStripItem)sender;
			ToolStripDropDownItem btnDraw = (ToolStripDropDownItem)((ToolStrip)ChartControl.Controls["tsrTool"]).Items["btnDraw"];
			
			foreach (ToolStripItem drawItem in btnDraw.DropDownItems)
			{
				if (item.Name.Replace("btn", string.Empty) == drawItem.Name)
				{
					drawItem.PerformClick();
					return;
				}
			}
		}
		
		private void btn1_Click(object sender, EventArgs e)
		{
			ToolStripItem item = (ToolStripItem)sender;
			ToolStripDropDownItem btnDraw = (ToolStripDropDownItem)((ToolStripDropDownItem)((ToolStrip)ChartControl.Controls["tsrTool"]).Items["btnDraw"]).DropDownItems["mnuChartMarker"];
			
			foreach (ToolStripItem drawItem in btnDraw.DropDownItems)
			{
				
				if (item.Name.Replace("btn", string.Empty) == drawItem.Name)
				{
					drawItem.PerformClick();
					return;
				}
			}
		}
		
				
		private void btn2_Click(object sender, EventArgs e)
		{
			ToolStripItem item = (ToolStripItem)sender;
			ToolStripDropDownItem btnDraw = (ToolStripDropDownItem)((ToolStripDropDownItem)((ToolStrip)ChartControl.Controls["tsrTool"]).Items["btnDraw"]).DropDownItems["mnuSnapMode"];
			
			foreach (ToolStripItem drawItem in btnDraw.DropDownItems)
			{
				
				if (item.Name.Replace("btn", string.Empty) == drawItem.Name)
				{
					drawItem.PerformClick();
					return;
				}
			}
		}
		
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            
        }
		
		protected override void OnTermination()
		{
			if (ChartControl == null) return;
			if (!showSnapMode && !showLines && !showFibs && !showChannels && !showShapes && !showMarkers) return;
			
			if (ts != null) 
			{
				ts.Items.Clear();
				ChartControl.Controls.Remove(ts);
				ts.Dispose();
				ts = null;
			}
		}

        #region Properties
        [Description("")]
        [GridCategory("Parameters")]
		public bool ShowSnapMode
		{
			get{ return showSnapMode;}
			set{showSnapMode = value;}
		}
		
		[Description("")]
        [GridCategory("Parameters")]
		public bool ShowLines
		{
			get{ return showLines;}
			set{showLines = value;}
		}
		
		[Description("")]
        [GridCategory("Parameters")]
		public bool ShowFibs
		{
			get{ return showFibs;}
			set{showFibs = value;}
		}
		
		[Description("")]
        [GridCategory("Parameters")]
		public bool ShowChannels
		{
			get{ return showChannels;}
			set{showChannels = value;}
		}
		
		[Description("")]
        [GridCategory("Parameters")]
		public bool ShowShapes
		{
			get{ return showShapes;}
			set{showShapes = value;}
		}
		
		[Description("")]
        [GridCategory("Parameters")]
		public bool ShowMarkers
		{
			get{ return showMarkers;}
			set{showMarkers = value;}
		}
	
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ToolbarShortcutV1[] cacheToolbarShortcutV1 = null;

        private static ToolbarShortcutV1 checkToolbarShortcutV1 = new ToolbarShortcutV1();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public ToolbarShortcutV1 ToolbarShortcutV1(bool showChannels, bool showFibs, bool showLines, bool showMarkers, bool showShapes, bool showSnapMode)
        {
            return ToolbarShortcutV1(Input, showChannels, showFibs, showLines, showMarkers, showShapes, showSnapMode);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public ToolbarShortcutV1 ToolbarShortcutV1(Data.IDataSeries input, bool showChannels, bool showFibs, bool showLines, bool showMarkers, bool showShapes, bool showSnapMode)
        {
            if (cacheToolbarShortcutV1 != null)
                for (int idx = 0; idx < cacheToolbarShortcutV1.Length; idx++)
                    if (cacheToolbarShortcutV1[idx].ShowChannels == showChannels && cacheToolbarShortcutV1[idx].ShowFibs == showFibs && cacheToolbarShortcutV1[idx].ShowLines == showLines && cacheToolbarShortcutV1[idx].ShowMarkers == showMarkers && cacheToolbarShortcutV1[idx].ShowShapes == showShapes && cacheToolbarShortcutV1[idx].ShowSnapMode == showSnapMode && cacheToolbarShortcutV1[idx].EqualsInput(input))
                        return cacheToolbarShortcutV1[idx];

            lock (checkToolbarShortcutV1)
            {
                checkToolbarShortcutV1.ShowChannels = showChannels;
                showChannels = checkToolbarShortcutV1.ShowChannels;
                checkToolbarShortcutV1.ShowFibs = showFibs;
                showFibs = checkToolbarShortcutV1.ShowFibs;
                checkToolbarShortcutV1.ShowLines = showLines;
                showLines = checkToolbarShortcutV1.ShowLines;
                checkToolbarShortcutV1.ShowMarkers = showMarkers;
                showMarkers = checkToolbarShortcutV1.ShowMarkers;
                checkToolbarShortcutV1.ShowShapes = showShapes;
                showShapes = checkToolbarShortcutV1.ShowShapes;
                checkToolbarShortcutV1.ShowSnapMode = showSnapMode;
                showSnapMode = checkToolbarShortcutV1.ShowSnapMode;

                if (cacheToolbarShortcutV1 != null)
                    for (int idx = 0; idx < cacheToolbarShortcutV1.Length; idx++)
                        if (cacheToolbarShortcutV1[idx].ShowChannels == showChannels && cacheToolbarShortcutV1[idx].ShowFibs == showFibs && cacheToolbarShortcutV1[idx].ShowLines == showLines && cacheToolbarShortcutV1[idx].ShowMarkers == showMarkers && cacheToolbarShortcutV1[idx].ShowShapes == showShapes && cacheToolbarShortcutV1[idx].ShowSnapMode == showSnapMode && cacheToolbarShortcutV1[idx].EqualsInput(input))
                            return cacheToolbarShortcutV1[idx];

                ToolbarShortcutV1 indicator = new ToolbarShortcutV1();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ShowChannels = showChannels;
                indicator.ShowFibs = showFibs;
                indicator.ShowLines = showLines;
                indicator.ShowMarkers = showMarkers;
                indicator.ShowShapes = showShapes;
                indicator.ShowSnapMode = showSnapMode;
                Indicators.Add(indicator);
                indicator.SetUp();

                ToolbarShortcutV1[] tmp = new ToolbarShortcutV1[cacheToolbarShortcutV1 == null ? 1 : cacheToolbarShortcutV1.Length + 1];
                if (cacheToolbarShortcutV1 != null)
                    cacheToolbarShortcutV1.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheToolbarShortcutV1 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ToolbarShortcutV1 ToolbarShortcutV1(bool showChannels, bool showFibs, bool showLines, bool showMarkers, bool showShapes, bool showSnapMode)
        {
            return _indicator.ToolbarShortcutV1(Input, showChannels, showFibs, showLines, showMarkers, showShapes, showSnapMode);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.ToolbarShortcutV1 ToolbarShortcutV1(Data.IDataSeries input, bool showChannels, bool showFibs, bool showLines, bool showMarkers, bool showShapes, bool showSnapMode)
        {
            return _indicator.ToolbarShortcutV1(input, showChannels, showFibs, showLines, showMarkers, showShapes, showSnapMode);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ToolbarShortcutV1 ToolbarShortcutV1(bool showChannels, bool showFibs, bool showLines, bool showMarkers, bool showShapes, bool showSnapMode)
        {
            return _indicator.ToolbarShortcutV1(Input, showChannels, showFibs, showLines, showMarkers, showShapes, showSnapMode);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.ToolbarShortcutV1 ToolbarShortcutV1(Data.IDataSeries input, bool showChannels, bool showFibs, bool showLines, bool showMarkers, bool showShapes, bool showSnapMode)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ToolbarShortcutV1(input, showChannels, showFibs, showLines, showMarkers, showShapes, showSnapMode);
        }
    }
}
#endregion
